@@ -19,9 +19,13 @@ import os
 import platform
 import stat
 import sys
+import sysconfig
 import time
+import traceback
 from collections import namedtuple
 from copy import copy
+from dataclasses import dataclass
+from enum import Enum
 from stat import S_IFLNK
 from typing import Union, Optional, Any, AnyStr, overload, cast
 
@@ -39,6 +43,13 @@ PERM_DEF = 0o777  # Default permission bits.
 PERM_DEF_FILE = 0o666  # Default permission bits (regular file)
 PERM_ALL = 0o7777  # All permission bits.
 
+STDLIB_PATH = os.path.realpath(sysconfig.get_path("stdlib"))
+PYFAKEFS_PATH = os.path.dirname(__file__)
+PYFAKEFS_TEST_PATHS = [
+    os.path.join(PYFAKEFS_PATH, "tests"),
+    os.path.join(PYFAKEFS_PATH, "pytest_tests"),
+]
+
 _OpenModes = namedtuple(
     "_OpenModes",
     "must_exist can_read can_write truncate append must_not_exist",
@@ -196,6 +207,42 @@ def matching_string(  # type: ignore[misc]
     return string  # pytype: disable=bad-return-type
 
 
+@dataclass
+class FSProperties:
+    sep: str
+    altsep: Optional[str]
+    pathsep: str
+    linesep: str
+    devnull: str
+
+
+# pure POSIX file system properties, for use with PosixPath
+POSIX_PROPERTIES = FSProperties(
+    sep="/",
+    altsep=None,
+    pathsep=":",
+    linesep="\n",
+    devnull="/dev/null",
+)
+
+# pure Windows file system properties, for use with WindowsPath
+WINDOWS_PROPERTIES = FSProperties(
+    sep="\\",
+    altsep="/",
+    pathsep=";",
+    linesep="\r\n",
+    devnull="NUL",
+)
+
+
+class FSType(Enum):
+    """Defines which file system properties to use."""
+
+    DEFAULT = 0  # use current OS file system + modifications in fake file system
+    POSIX = 1  # pure POSIX properties, for use in PosixPath
+    WINDOWS = 2  # pure Windows properties, for use in WindowsPath
+
+
 class FakeStatResult:
     """Mimics os.stat_result for use as return type of `stat()` and similar.
     This is needed as `os.stat_result` has no possibility to set
@@ -429,3 +476,66 @@ class TextBufferIO(io.TextIOWrapper):
 
     def putvalue(self, value: bytes) -> None:
         self._bytestream.write(value)
+
+
+def is_called_from_skipped_module(
+    skip_names: list, case_sensitive: bool, check_open_code: bool = False
+) -> bool:
+    def starts_with(path, string):
+        if case_sensitive:
+            return path.startswith(string)
+        return path.lower().startswith(string.lower())
+
+    # in most cases we don't have skip names and won't need the overhead
+    # of analyzing the traceback, except when checking for open_code
+    if not skip_names and not check_open_code:
+        return False
+
+    stack = traceback.extract_stack()
+
+    # handle the case that we try to call the original `open_code`
+    # (since Python 3.12)
+    # The stack in this case is:
+    # -1: helpers.is_called_from_skipped_module: 'stack = traceback.extract_stack()'
+    # -2: fake_open.fake_open: 'if is_called_from_skipped_module('
+    # -3: fake_io.open: 'return fake_open('
+    # -4: fake_io.open_code : 'return self._io_module.open_code(path)'
+    if (
+        check_open_code
+        and stack[-4].name == "open_code"
+        and stack[-4].line == "return self._io_module.open_code(path)"
+    ):
+        return True
+
+    if not skip_names:
+        return False
+
+    caller_filename = next(
+        (
+            frame.filename
+            for frame in stack[::-1]
+            if not frame.filename.startswith("<frozen ")
+            and not starts_with(frame.filename, STDLIB_PATH)
+            and (
+                not starts_with(frame.filename, PYFAKEFS_PATH)
+                or any(
+                    starts_with(frame.filename, test_path)
+                    for test_path in PYFAKEFS_TEST_PATHS
+                )
+            )
+        ),
+        None,
+    )
+
+    if caller_filename:
+        caller_module_name = os.path.splitext(caller_filename)[0]
+        caller_module_name = caller_module_name.replace(os.sep, ".")
+
+        if any(
+            [
+                caller_module_name == sn or caller_module_name.endswith("." + sn)
+                for sn in skip_names
+            ]
+        ):
+            return True
+    return False
